import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/routines/bloc/routine_bloc/routine_bloc.dart';
import 'package:syncrow_web/pages/routines/view/create_new_routine_view.dart';
import 'package:syncrow_web/pages/routines/widgets/main_routine_view/fetch_routine_scenes_automation.dart';
import 'package:syncrow_web/pages/routines/widgets/main_routine_view/routine_view_card.dart';
import 'package:syncrow_web/pages/space_tree/bloc/space_tree_bloc.dart';
import 'package:syncrow_web/pages/space_tree/view/space_tree_view.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/snack_bar.dart';

class RoutinesView extends StatefulWidget {
  const RoutinesView({super.key});

  @override
  State<RoutinesView> createState() => _RoutinesViewState();
}

class _RoutinesViewState extends State<RoutinesView> {
  @override
  void initState() {
    super.initState();
    context.read<RoutineBloc>().add(FetchDevicesInRoutine());
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<RoutineBloc, RoutineState>(
      builder: (context, state) {
        if (state.createRoutineView) {
          return const CreateNewRoutineView();
        }
        return Row(
          children: [
            Expanded(
                child:
                    // SideSpacesView(
                    //   onSelectAction: (String communityId, String spaceId) {
                    //     // context.read<RoutineBloc>()
                    //     //   ..add(LoadScenes(spaceId, communityId))
                    //     //   ..add(LoadAutomation(spaceId));
                    //   },
                    // )
                    SpaceTreeView(
              onSelect: () {},
            )),
            Expanded(
              flex: 3,
              child: Padding(
                padding: const EdgeInsets.all(16),
                child: Row(
                  children: [
                    Column(
                      mainAxisSize: MainAxisSize.min,
                      crossAxisAlignment: CrossAxisAlignment.start,
                      mainAxisAlignment: MainAxisAlignment.start,
                      children: [
                        Text(
                          "Create New Routines",
                          style: Theme.of(context).textTheme.titleLarge?.copyWith(
                                color: ColorsManager.grayColor,
                                fontWeight: FontWeight.bold,
                              ),
                        ),
                        const SizedBox(
                          height: 10,
                        ),
                        RoutineViewCard(
                          onTap: () {
                            if (context.read<SpaceTreeBloc>().selectedCommunityId.isNotEmpty &&
                                context.read<SpaceTreeBloc>().selectedSpaceId.isNotEmpty) {
                              context.read<RoutineBloc>().add(
                                    (ResetRoutineState()),
                                  );
                              BlocProvider.of<RoutineBloc>(context).add(
                                const CreateNewRoutineViewEvent(createRoutineView: true),
                              );
                            } else {
                              CustomSnackBar.redSnackBar('Please select a space');
                            }
                          },
                          icon: Icons.add,
                          textString: '',
                        ),
                        const SizedBox(
                          height: 15,
                        ),
                        const Expanded(child: FetchRoutineScenesAutomation()),
                      ],
                    ),
                  ],
                ),
              ),
            ),
          ],
        );
      },
    );
  }
}
