import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/constants/assets.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';
import 'package:syncrow_web/utils/helpers/responsice_layout_helper/responsive_layout_helper.dart';

class RoutineViewCard extends StatelessWidget with HelperResponsiveLayout {
  const RoutineViewCard({
    super.key,
    required this.onTap,
    required this.icon,
    required this.textString,
    this.isFromScenes,
    this.iconInBytes,
  });

  final Function() onTap;
  final dynamic icon;
  final String textString;
  final bool? isFromScenes;
  final Uint8List? iconInBytes;

  @override
  Widget build(BuildContext context) {
    final double cardWidth = isSmallScreenSize(context)
        ? 120
        : isMediumScreenSize(context)
            ? 135
            : 150;

    final double cardHeight = isSmallScreenSize(context) ? 160 : 170;

    final double iconSize = isSmallScreenSize(context)
        ? 50
        : isMediumScreenSize(context)
            ? 60
            : 70;

    return ConstrainedBox(
      constraints: BoxConstraints(
        maxWidth: cardWidth,
        maxHeight: cardHeight,
      ),
      child: Card(
        elevation: 3,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(10),
        ),
        color: ColorsManager.whiteColors,
        child: InkWell(
          onTap: onTap,
          borderRadius: BorderRadius.circular(10),
          child: Padding(
            padding: const EdgeInsets.all(8.0),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: [
                Center(
                  child: Container(
                    decoration: BoxDecoration(
                      color: ColorsManager.graysColor,
                      borderRadius: BorderRadius.circular(120),
                      border: Border.all(
                        color: ColorsManager.greyColor,
                        width: 2.0,
                      ),
                    ),
                    height: iconSize,
                    width: iconSize,
                    child: (isFromScenes ?? false)
                        ? (iconInBytes != null && iconInBytes?.isNotEmpty == true)
                            ? Image.memory(
                                iconInBytes!,
                                height: iconSize,
                                width: iconSize,
                                fit: BoxFit.contain,
                                errorBuilder: (context, error, stackTrace) => Image.asset(
                                  Assets.logo,
                                  height: iconSize,
                                  width: iconSize,
                                  fit: BoxFit.contain,
                                ),
                              )
                            : Image.asset(
                                Assets.logo,
                                height: iconSize,
                                width: iconSize,
                                fit: BoxFit.contain,
                              )
                        : (icon is String && icon.endsWith('.svg'))
                            ? SvgPicture.asset(
                                icon,
                                fit: BoxFit.contain,
                              )
                            : Icon(
                                icon,
                                color: ColorsManager.dialogBlueTitle,
                                size: isSmallScreenSize(context) ? 30 : 40,
                              ),
                  ),
                ),
                const SizedBox(height: 8),
                Padding(
                  padding: const EdgeInsets.symmetric(horizontal: 3),
                  child: Text(
                    textString,
                    textAlign: TextAlign.center,
                    overflow: TextOverflow.ellipsis,
                    maxLines: 2,
                    style: context.textTheme.bodySmall?.copyWith(
                      color: ColorsManager.blackColor,
                      fontSize: isSmallScreenSize(context) ? 10 : 12,
                    ),
                  ),
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}
