import 'package:syncrow_web/pages/spaces_management/all_spaces/model/base_tag.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/product_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/create_space_template_body_model.dart';
import 'package:uuid/uuid.dart';

class TagModel extends BaseTag {
  TagModel({
    String? uuid,
    required String? tag,
    ProductModel? product,
    String? internalId,
    String? location,
  }) : super(
          uuid: uuid,
          tag: tag,
          product: product,
          internalId: internalId,
          location: location,
        );
  factory TagModel.fromJson(Map<String, dynamic> json) {
    final String internalId = json['internalId'] ?? const Uuid().v4();

    return TagModel(
      uuid: json['uuid'] ,
      internalId: internalId,
      tag: json['tag'] ?? '',
      product: json['product'] != null
          ? ProductModel.fromMap(json['product'])
          : null,
    );
  }

  @override
  TagModel copyWith(
      {String? tag,
      ProductModel? product,
      String? uuid,
      String? location,
      String? internalId}) {
    return TagModel(
      tag: tag ?? this.tag,
      product: product ?? this.product,
      location: location ?? this.location,
      internalId: internalId ?? this.internalId,
      uuid:uuid?? this.uuid
    );
  }

  Map<String, dynamic> toJson() {
    return {
      'uuid': uuid,
      'tag': tag,
      'product': product?.toMap(),
    };
  }
}

extension TagModelExtensions on TagModel {
  TagBodyModel toTagBodyModel() {
    return TagBodyModel()
      ..uuid = uuid
      ..tag = tag ?? ''
      ..productUuid = product?.uuid;
  }
}
