import 'package:syncrow_web/pages/spaces_management/space_model/models/create_space_template_body_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/services/api/http_service.dart';
import 'package:syncrow_web/utils/constants/api_const.dart';
import 'package:syncrow_web/utils/constants/temp_const.dart';

class SpaceModelManagementApi {
  Future<List<SpaceTemplateModel>> listSpaceModels({int page = 1}) async {
    final response = await HTTPService().get(
      path: ApiEndpoints.listSpaceModels
          .replaceAll('{projectId}', TempConst.projectId),
      queryParameters: {'page': page},
      expectedResponseModel: (json) {
        List<dynamic> jsonData = json['data'];
        return jsonData.map((jsonItem) {
          return SpaceTemplateModel.fromJson(jsonItem);
        }).toList();
      },
    );
    return response;
  }

  Future<SpaceTemplateModel?> createSpaceModel(
      CreateSpaceTemplateBodyModel spaceModel) async {
    final response = await HTTPService().post(
      path: ApiEndpoints.createSpaceModel
          .replaceAll('{projectId}', TempConst.projectId),
      showServerMessage: true,
      body: spaceModel.toJson(),
      expectedResponseModel: (json) {
        return SpaceTemplateModel.fromJson(json['data']);
      },
    );
    return response;
  }


  Future<String?> updateSpaceModel(
      CreateSpaceTemplateBodyModel spaceModel, String spaceModelUuid) async {
    final response = await HTTPService().put(
      path: ApiEndpoints.updateSpaceModel
          .replaceAll('{projectId}', TempConst.projectId).replaceAll('{spaceModelUuid}', spaceModelUuid),
      body: spaceModel.toJson(),
      expectedResponseModel: (json) {
        return json['message'];
      },
    );
    return response;
  }

  Future<SpaceTemplateModel?> getSpaceModel(String spaceModelUuid) async {
    final response = await HTTPService().get(
      path: ApiEndpoints.getSpaceModel
          .replaceAll('{projectId}', TempConst.projectId)
          .replaceAll('{spaceModelUuid}', spaceModelUuid),
      showServerMessage: true,
      expectedResponseModel: (json) {
        return SpaceTemplateModel.fromJson(json['data']);
      },
    );
    return response;
  }
}
